# API Docs

## Extrema

### minbytimebatch *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#windows">Window</a>)*

<p style="word-wrap: break-word">Calculates the minimum value of a specified attribute within a time window, and emits it.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:minbytimebatch(<INT|FLOAT|DOUBLE|LONG|STRING> attribute, <INT|LONG> time.batch.length)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the minimum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>FLOAT<br>DOUBLE<br>LONG<br>STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">time.batch.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the time window observed.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream cseEventStream (symbol string, price float, volume int);

from cseEventStream#window.extrema:minByTimeBatch(price, 1 sec) select symbol,price,volume
insert into outputStream ;
```
<p style="word-wrap: break-word">In the given example query, a time-batch window of 1 second is considered. After every second, the window will be reset, emitting the event with the minimum price.</p>

### minByLength *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#windows">Window</a>)*

<p style="word-wrap: break-word">The event with the minimum value for the given attribute in the specified sliding window is emitted.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:minByLength(<INT|FLOAT|DOUBLE|LONG|STRING> attribute, <INT|LONG> sliding.window.length)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the minimum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>FLOAT<br>DOUBLE<br>LONG<br>STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">sliding.window.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the sliding window observed.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream cseEventStream (symbol string, price float, volume int);
from cseEventStream#window.extrema:minByLength(price, 4) select symbol,price,volume
insert into outputStream ;
```
<p style="word-wrap: break-word">In the given example query, a sliding window of length 4 is considered. The event with the minimum price will be emitted out.</p>

### minByLengthBatch *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#windows">Window</a>)*

<p style="word-wrap: break-word">Calculates the minimum value of a specified attribute inside a batch window and emits it.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:minByLengthBatch(<INT|FLOAT|DOUBLE|LONG|STRING> attribute, <INT> batch.length)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the minimum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>FLOAT<br>DOUBLE<br>LONG<br>STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">batch.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the batch involved.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream cseEventStream (symbol string, price float, volume int);

from cseEventStream#window.extrema:minByLengthBatch(price, 4) select symbol,price,volume
insert into outputStream ;
```
<p style="word-wrap: break-word">In the given example query, a batch of 4 events will be collected. Once the window is full, the item with the minimum price in the batch will be emitted out and the window will be reset.</p>

### maxByLength *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#windows">Window</a>)*

<p style="word-wrap: break-word">The event with the maximum value for the given attribute in the specified sliding window is emitted.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:maxByLength(<INT|FLOAT|DOUBLE|LONG|STRING> attribute, <INT|LONG> sliding.window.length)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the maximum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>FLOAT<br>DOUBLE<br>LONG<br>STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">sliding.window.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the sliding window observed.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream cseEventStream (symbol string, price float, volume int);
from cseEventStream#window.extrema:maxByLength(price, 4) select symbol,price,volume
insert into outputStream ;
```
<p style="word-wrap: break-word">In the given example query, a sliding window of length 4 is considered. The event with the maximum price will be emitted out.</p>

### maxbytimebatch *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#windows">Window</a>)*

<p style="word-wrap: break-word">Calculates the maximum value of a specified attribute within a time window, and emits it.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:maxbytimebatch(<INT|FLOAT|DOUBLE|LONG|STRING> attribute, <INT|LONG> time.batch.length)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the maximum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>FLOAT<br>DOUBLE<br>LONG<br>STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">time.batch.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the time window observed.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream cseEventStream (symbol string, price float, volume int);
from cseEventStream#window.extrema:maxByTimeBatch(price, 1 sec) select symbol,price,volume
insert into outputStream ;
```
<p style="word-wrap: break-word">In the given example query, a time-batch window of 1 second is considered. After every second, the window will be reset, emitting the event with the maximum price.</p>

### minbytime *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#windows">Window</a>)*

<p style="word-wrap: break-word">Calculates the minimum value of a specified attribute within a sliding time window and emits it. The output is updated for every event arrival and expiry during the time.window.length specified.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:minbytime(<INT|FLOAT|DOUBLE|LONG|STRING> attribute, <INT|LONG> time.window.length)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the minimum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>FLOAT<br>DOUBLE<br>LONG<br>STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">time.window.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the sliding time window observed.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream cseEventStream (symbol string, price float, volume int);
from cseEventStream#window.extrema:minByTime(price, 1 sec) select symbol,price,volume
insert into outputStream;
```
<p style="word-wrap: break-word">In the given example query, a sliding time window of 1 second is considered. This output is updated for every event arrival and expiry of every event (after 1 second of its arrival).</p>

### maxbytime *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#windows">Window</a>)*

<p style="word-wrap: break-word">Calculates the maximum value of a specified attribute within a sliding time window and emits it. The output is updated for every event arrival and expiry during the time.window.length specified.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:maxbytime(<INT|FLOAT|DOUBLE|LONG|STRING> attribute, <INT|LONG> time.window.length)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the maximum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>FLOAT<br>DOUBLE<br>LONG<br>STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">time.window.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the sliding time window observed.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream cseEventStream (symbol string, price float, volume int);
from cseEventStream#window.extrema:maxByTime(price, 1 sec) select symbol,price,volume
insert into outputStream ;
```
<p style="word-wrap: break-word">In the given example query, a sliding time window of 1 second is considered. This output is updated for every event arrival and expiry of every event (after 1 second of its arrival).</p>

### maxByLengthBatch *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#windows">Window</a>)*

<p style="word-wrap: break-word">Calculates the maximum value of a specified attribute inside a batch window and emits it.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:maxByLengthBatch(<INT|FLOAT|DOUBLE|LONG|STRING> attribute, <INT> batch.length)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the maximum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>FLOAT<br>DOUBLE<br>LONG<br>STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">batch.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the batch involved.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream cseEventStream (symbol string, price float, volume int);

from cseEventStream#window.extrema:maxByLengthBatch(price, 4) 
select symbol,price,volume
insert into outputStream ;
```
<p style="word-wrap: break-word">In the given example query, a batch of 4 events will be collected. Once the window is full, the item with the maximum price in the batch will be emitted out and the window will be reset.</p>

### kernelMinMax *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#stream-processors">Stream Processor</a>)*

<p style="word-wrap: break-word">kernalMinMax uses Gaussian Kernel to smooth the time series values in the given window size, and then determine the maxima and minima of that set of values. Returns the events with the minimum and/or maximum for the specified attribute within the given window length, with the extrema type as min or max as relevant.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:kernelMinMax(<INT|FLOAT|DOUBLE|LONG> attribute, <DOUBLE> bandwidth, <INT> window.size, <STRING> extrema.type)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the minimum and/or maximum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>FLOAT<br>DOUBLE<br>LONG</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">bandwidth</td>
        <td style="vertical-align: top; word-wrap: break-word">The bandwidth of the Gaussian Kernel calculation.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">DOUBLE</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">window.size</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the window within which the minimum and/or the maximum value for the given window should be identified.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">extrema.type</td>
        <td style="vertical-align: top; word-wrap: break-word">This can be min, max or minmax.min: If this is specified, minimum values are identified within the given window length, and they are returned with min as their extrema type. max: If this is specified, maximum values are identified within the given window length, and they are returned with max as their extrema type. minmax: If this is specified, both minimum and maximum values are identified within the given window length and returned. The extrema type is specified as min for the minimum events, and as max for the maximum events.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream inputStream (item string, price long);

from inputStream#extrema:kernelMinMax(price, 3, 7, ‘max’)
select *
insert into outputStream;
```
<p style="word-wrap: break-word">This query returns the maximum values for a set of price values.</p>

### topK *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#stream-processors">Stream Processor</a>)*

<p style="word-wrap: break-word">topK counts the frequency of different values of a specified attribute, and emits the highest (k) number of frequency values.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:topK(<INT|LONG|FLOAT|DOUBLE|STRING|BOOL|OBJECT> attribute, <INT> k.value)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the frequency is counted.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG<br>FLOAT<br>DOUBLE<br>STRING<br>BOOL<br>OBJECT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">k.value</td>
        <td style="vertical-align: top; word-wrap: break-word">The number of top frequencies required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream inputStream (item string, price long);

from inputStream#extrema:topK(item, 3)
insert all events into outputStream;)
```
<p style="word-wrap: break-word">In the given example query, the three items with the highest frequency counts will be emitted</p>

### bottomKTimeBatch *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#stream-processors">Stream Processor</a>)*

<p style="word-wrap: break-word">bottomKTimeBatch counts the frequency of different values of a specified attribute inside a time window, and emits the lowest (k) number of frequency values. Events are emitted only if there is a change in the bottomK results for each received chunk of events.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:bottomKTimeBatch(<INT|LONG|FLOAT|DOUBLE|STRING|BOOL|OBJECT> attribute, <INT|LONG> time.window, <INT> k.value)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the frequency is counted.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG<br>FLOAT<br>DOUBLE<br>STRING<br>BOOL<br>OBJECT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">time.window</td>
        <td style="vertical-align: top; word-wrap: break-word">The time window during which the frequency should be calculated.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">k.value</td>
        <td style="vertical-align: top; word-wrap: break-word">The number of bottom frequencies required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream inputStream (item string, price long);
from inputStream#extrema:bottomKTimeBatch(item, 1 sec,  3)
insert all events into outputStream;)
```
<p style="word-wrap: break-word">In the given example query, a batch of 1 second will be collected. Once the window is full, the 3 items with the lowest frequency will be emitted out and the window will be reset.</p>

### topKTimeBatch *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#stream-processors">Stream Processor</a>)*

<p style="word-wrap: break-word">topKTimeBatch counts the frequency of different values of a specified attribute inside a time window, and emits the highest (k) number of frequency values.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:topKTimeBatch(<INT|LONG|FLOAT|DOUBLE|STRING|BOOL|OBJECT> attribute, <INT|LONG> time.window, <INT> k.value)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the frequency is counted.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG<br>FLOAT<br>DOUBLE<br>STRING<br>BOOL<br>OBJECT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">time.window</td>
        <td style="vertical-align: top; word-wrap: break-word">The time window during which the frequency should be calculated.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">k.value</td>
        <td style="vertical-align: top; word-wrap: break-word">The number of top frequencies required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream inputStream (item string, price long);
from inputStream#extrema:topKTimeBatch(item, 1 sec,  3)
insert all events into outputStream;
```
<p style="word-wrap: break-word">In the given example query, a batch of 1 second will be collected. Once the window is full, the 3 items with the highest frequency will be emitted out and the window will be reset.</p>

### bottomK *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#stream-processors">Stream Processor</a>)*

<p style="word-wrap: break-word">bottomK counts the frequency of different values of a specified attribute, and emits the lowest (k) number of frequency values. Events are emitted only if there is a change in the bottomK results for each received chunk of events.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:bottomK(<INT|LONG|FLOAT|DOUBLE|STRING|BOOL|OBJECT> attribute, <INT> k.value)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the frequency is counted.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG<br>FLOAT<br>DOUBLE<br>STRING<br>BOOL<br>OBJECT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">k.value</td>
        <td style="vertical-align: top; word-wrap: break-word">The number of bottom frequencies required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream inputStream (item string, price long);

from inputStream#extrema:bottomK(item, 3)
insert all events into outputStream;)
```
<p style="word-wrap: break-word">In the given example query, the three items with the lowest frequency counts will be emitted</p>

### minMax *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#stream-processors">Stream Processor</a>)*

<p style="word-wrap: break-word">minMax finds the minimum and/or the maximum value within a given length window (maxPreBound+maxPostBound), where following conditions are met. <br><br>For minimum: <br>An event where the value for the specified attribute is greater by the percentage specified as the preBoundChange should have arrived prior to the event with the minimum value, within the maxPreBound length window.<br>An event where the value for the specified attribute is greater by the percentage specified as the postBoundChange should have arrived after the event with the minimum value, within the maxPostBound length window.<br><br>For maximum: <br>An event where the value for the specified attribute is less by the percentage specified as the preBoundChange should have arrived prior to the event with the maximum value, within the maxPreBound length window.<br>An event where the value for the specified attribute is less by the percentage specified as the postBoundChange should have arrived after the event with the maximum value, within the maxPostBound length window.<br><br>Returns the events with the minimum and/or maximum for the specified attribute within the given window length, with the extrema type as min or max as relevant. These events are returned with the following additional parameters.<br>preBound: The actual distance between the minimum/maximum value and the threshold value. This value should be within the MaxPreBound window.<br>postBound: The actual distance between the minimum/maximum value and the threshold value. This value should be within the MaxPostBound window.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:minMax(<INT|FLOAT|DOUBLE|LONG> attribute, <INT> max.pre.bound, <INT> max.post.bound, <DOUBLE> pre.bound.change, <DOUBLE> post.bound.change, <STRING> extrema.type)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the minimum and/or maximum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>FLOAT<br>DOUBLE<br>LONG</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">max.pre.bound</td>
        <td style="vertical-align: top; word-wrap: break-word">The maximum pre window length to be considered (before the min/max event).</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">max.post.bound</td>
        <td style="vertical-align: top; word-wrap: break-word">The maximum post window length to be considered (after the min/max event).</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">pre.bound.change</td>
        <td style="vertical-align: top; word-wrap: break-word">The threshold, pre-bound change percentage.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">DOUBLE</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">post.bound.change</td>
        <td style="vertical-align: top; word-wrap: break-word">The threshold, post-bound change percentage.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">DOUBLE</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">extrema.type</td>
        <td style="vertical-align: top; word-wrap: break-word">This can be min, max or minmax.<br>min: If this is specified, minimum values are identified within the given window length, and they are returned with min as their extrema type.<br>max: If this is specified, maximum values are identified within the given window length, and they are returned with max as their extrema type.<br>minmax: If this is specified, both minimum and maximum values are identified within the given window length and returned. The extrema type is specified as min for the minimum events, and as max for the maximum events.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
from inputStream#extrema:minMax(price, 4, 4, 1, 2, 'max')
select *
insert into outputStream; 
```
<p style="word-wrap: break-word">Returns the maximum values found within a set of price values.</p>

<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
from inputStream#extrema:minMax(price, 4, 4, 1, 2, 'min')
select *
insert into outputStream; 
```
<p style="word-wrap: break-word">returns the minimum values found within a set of price values.</p>

<span id="example-3" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 3</span>
```
from inputStream#extrema:minMax(price, 4, 4, 1, 2, 'minmax')
select *
insert into outputStream; 
```
<p style="word-wrap: break-word">Returns both the minimum values and the maximum values found within a set of price values.</p>

### topKLengthBatch *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#stream-processors">Stream Processor</a>)*

<p style="word-wrap: break-word">topKLengthBatch counts the frequency of different values of a specified attribute inside a batch window, and emits the highest (k) number of frequency values.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:topKLengthBatch(<INT|LONG|FLOAT|DOUBLE|STRING|BOOL|OBJECT> attribute, <INT> window.length, <INT> k.value)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the frequency is counted.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG<br>FLOAT<br>DOUBLE<br>STRING<br>BOOL<br>OBJECT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">window.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the window.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">k.value</td>
        <td style="vertical-align: top; word-wrap: break-word">The number of top frequencies required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream inputStream (item string, price long);

from inputStream#extrema:topKLengthBatch(item, 6, 3)
insert all events into outputStream;)
```
<p style="word-wrap: break-word">In the given example query, a batch of 6 events will be collected. Once the window is full, the 3 items with the highest frequency will be emitted out and the window will be reset.</p>

### kalmanMinMax *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#stream-processors">Stream Processor</a>)*

<p style="word-wrap: break-word">The kalmanMinMax function uses the kalman filter to smooth the time series values in the given window size, and then determine the maxima and minima of that set of values. Returns the events with the minimum and/or maximum for the specified attribute within the given window length, with the extrema type as min or max as relevant.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:kalmanMinMax(<INT|LONG|FLOAT|DOUBLE> attribute, <DOUBLE> q, <DOUBLE> r, <INT> window.size, <STRING> extrema.type)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the minimum and/or maximum value is required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG<br>FLOAT<br>DOUBLE</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">q</td>
        <td style="vertical-align: top; word-wrap: break-word">The standard deviation of the process noise.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">DOUBLE</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">r</td>
        <td style="vertical-align: top; word-wrap: break-word">The standard deviation of the measurement noise.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">DOUBLE</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">window.size</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the window within which the minimum and/or the maximum value for the given window should be identified.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">extrema.type</td>
        <td style="vertical-align: top; word-wrap: break-word">This can be min, max or minmax.min: If this is specified, minimum values are identified within the given window length, and they are returned with min as their extrema type. max: If this is specified, maximum values are identified within the given window length, and they are returned with max as their extrema type. minmax: If this is specified, both minimum and maximum values are identified within the given window length and returned. The extrema type is specified as min for the minimum events, and as max for the maximum events.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream inputStream (item string, price long);

from inputStream#extrema:kalmanMinMax(price, 0.000001,0.0001, 25, 'min')insert all events into outputStream;)
```
<p style="word-wrap: break-word">This query returns the minimum values for a set of price values.</p>

### bottomKLengthBatch *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#stream-processors">Stream Processor</a>)*

<p style="word-wrap: break-word">bottomKLengthBatch counts the frequency of different values of a specified attribute inside a batch window, and emits the lowest (k) number of frequency values. The bottom K frequency values are emitted per batch.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
extrema:bottomKLengthBatch(<INT|LONG|FLOAT|DOUBLE|STRING|BOOL|OBJECT> attribute, <INT> window.length, <INT> k.value)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">The attribute of which the frequency is counted.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>LONG<br>FLOAT<br>DOUBLE<br>STRING<br>BOOL<br>OBJECT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">window.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The length of the window.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">k.value</td>
        <td style="vertical-align: top; word-wrap: break-word">The number of bottom frequencies required.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream inputStream (item string, price long);

from inputStream#extrema:bottomKLengthBatch(item, 6, 3)
insert all events into outputStream;)
```
<p style="word-wrap: break-word">In the given example query, a batch of 6 events will be collected. Once the window is full, the 3 items with the lowest frequency will be emitted out and the window will be reset.</p>

